#!/bin/ksh
#---------------------------------------------------------------------+
#  Copyright (c)2001 Corey R Clingo. All Rights Reserved.             |
#                                                                     |
#                http://www.thecassandraproject.org/                  |
#                                                                     |
#  This is free software; you can redistribute it and/or modify it    |
#  under the terms of the GNU General Public License as published by  |
#  the Free Software Foundation; either version 2 of the License, or  |
#  (at your option) any later version.                                |
#                                                                     |
#  This software is distributed in the hope that it will be useful,   |
#  but WITHOUT ANY WARRANTY; without even the implied warranty of     |
#  merchantability or fitness for a particular purpose. See the GNU   |
#  General Public License (http://www.fsf.org/copyleft/gpl.htm)for    |
#  more details.                                                      |
#                                                                     |
#  To obtain the GNU General Public License, write to:                |
#                                                                     |
#      Free Software Foundation, Inc.                                 |
#      59 Temple Place - Suite 330                                    |
#      Boston, MA  02111-1307                                         |
#      USA                                                            |
#---------------------------------------------------------------------+
# kill_idle_dm.sh - simple script to kill idle DMs.
# 
# Uses a combination of PICK1 (the currently selected point) and ST0
# (the current display) to determine if any activity has taken place.
#
# This normally should be run in the background, i.e., start it with
#   nohup kill_idle_dm.sh 2>&1 >/dev/null &
# or something similar. You can run in foreground to experiment
#
# Also, this may be a little too aggressive in killing DMs (e.g., when
# you have a trend up and are watching it).

# ---------------------------------------------------------------------
# Configuration variables
# ---------------------------------------------------------------------
# Delay between PICK1/ST0 checks, sec
DELAY=300
               
# Number of times we check for a change in PICK1/ST0 before we kill a DM
DM_INACT_LIMIT=3

# Names of DMs to monitor
DMNAMES="XBDM02 XBDM03 XBDM04 XBDM05"

# What to call myself in the friendly message that the user gets as his DM
# is dying :)
MYNAME="The_DiM_Reaper"

# Location of "pref" command
PREF="/usr/local/pref"

# ------------ End of configuration variables -------------------------

i=1
for DM in $(echo $DMNAMES); do
    DM_PAST_STATE[$i]="$($PREF -${DM} PICK1) $($PREF -${DM} ST0)"
    DM_INACT_CNT[$i]=0
    i=$(($i+1))
done
sleep $DELAY

# TODO: figure out a check to see if the DM is even running, and ignore
# if not.  pref returns a value for ST0 even on non-running DMs.  This
# could also cause premature killing of a DM occasionally

while [ 0 ]; do
    i=1
    for DM in $(echo $DMNAMES); do
        DM_CURR_STATE[$i]="$($PREF -${DM} PICK1) $($PREF -${DM} ST0)"
        if [ "${DM_CURR_STATE[$i]}" = "${DM_PAST_STATE[$i]}" ]; then
            DM_INACT_CNT[$i]=$((${DM_INACT_CNT[$i]}+1))
        else
            DM_INACT_CNT[$i]=0
            DM_PAST_STATE[$i]=${DM_CURR_STATE[$i]}
        fi

        if [ ${DM_INACT_CNT[$i]} -gt $DM_INACT_LIMIT ]; then
            $PREF -${DM} DMCMD "dmexit 0 $MYNAME"
            DM_INACT_CNT[$i]=0
        fi

        i=$(($i+1))
    done

    sleep $DELAY
done 

# ---------------
# end script file
# ---------------
